# ======================================================================
# Define options
# ======================================================================
set opt(chan)   Channel/WirelessChannel
set opt(prop)   Propagation/TwoRayGround
set opt(netif)  Phy/WirelessPhy
set opt(mac)    Mac/802_11
set opt(ll)     LL
set opt(ant)    Antenna/OmniAntenna
set opt(ifqlen) 50                          ;# max packets in if-queue

# set defaults for commandline options 
set opt(x)      500                         ;# X dimension
set opt(y)      500                         ;# Y dimension
set opt(nn)     50                          ;# number of nodes
set opt(time)   100.0                       ;# simulation time
set opt(rp)     DSDV                        ;# routing protocol
set opt(tm)     ""                          ;# traffic model
set opt(mm)     ""                          ;# movement model
set opt(tr)     out.tr                      ;# trace file
set opt(nam)    ""                          ;# nam trace file

# ======================================================================
# Functions
# ======================================================================
proc usage {} {
    global argv0

    puts "\nusage: $argv0 \[-x width\] \[-y height\] \[-nn nodes\] "
    puts "         \[-time stoptime\] \[-rp protocol\] "
    puts "         -tm traffic model -mm movement model "
    puts "         \[-tr trace file\] \[-nam nam trace file\]\n"
}

proc getopt {argc argv} {
  global opt

  for {set i 0} {$i < $argc} {incr i} {
    set arg [lindex $argv $i]
    if {[string range $arg 0 0] != "-"} continue

    set name [string range $arg 1 end]
    set opt($name) [lindex $argv [expr $i+1]]
  }
}



# ======================================================================
# Main Program
# ======================================================================

# get commandline options
getopt $argc $argv
if { $opt(mm) == "" || $opt(tm) == "" } {
  usage
  exit
}

# set Priority Queue to prefer Routing
if { [string compare $opt(rp) "DSR"] == 0} { 
  set opt(ifq)    CMUPriQueue
} else {
  set opt(ifq)    Queue/DropTail/PriQueue
}
Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# create simulator instance
set ns_ [new Simulator]

# set wireless channel, radiomodel and topography objects
set wtopo [new Topography]

# create trace object for ns and nam
set tracefd [open $opt(tr) w]
$ns_ trace-all $tracefd
$ns_ use-newtrace
if { [string compare $opt(nam) ""] != 0} {
  set namtrace [open $opt(nam) w]
  $ns_ namtrace-all-wireless $namtrace $opt(x) $opt(y)
  $ns_ color 0 red
  $ns_ color 1 blue
}

# define topology
$wtopo load_flatgrid $opt(x) $opt(y)

# Create God
set god_ [create-god $opt(nn)]

# prepare wireless channel
set chan_1_ [new $opt(chan)]

# define how nodes should be created
$ns_ node-config -adhocRouting $opt(rp) \
                 -llType $opt(ll) \
                 -macType $opt(mac) \
                 -ifqType $opt(ifq) \
                 -ifqLen $opt(ifqlen) \
                 -antType $opt(ant) \
                 -propType $opt(prop) \
                 -phyType $opt(netif) \
                 -channel $chan_1_ \
                 -topoInstance $wtopo \
                 -agentTrace ON \
                 -routerTrace ON \
                 -macTrace OFF \
                 -movementTrace OFF

# Create the specified number of nodes
for {set i 0} {$i < $opt(nn) } {incr i} {
  set node_($i) [$ns_ node]
  $node_($i) random-motion 0 ;# disable random motion
  if {$opt(rp) == "NRLOLSR"} {
    [$node_($i) set ragent_] mac 1
  }
}

# load node movement model
source $opt(mm)

# load traffic model
source $opt(tm)


for {set i 0} {$i < $opt(nn) } {incr i} {
  # set node size in nam
  $ns_ initial_node_pos $node_($i) 15 ;
  # Tell nodes when the simulation ends
  $ns_ at $opt(time).000000001 "$node_($i) reset";
}

# prepare the stop procedure
proc stop {} {
  global ns_ tracefd namtrace opt
  $ns_ flush-trace
  close $tracefd
  if { [string compare $opt(nam) ""] != 0} {
    close $namtrace
  }
}

# tell ns the simulation stop time
$ns_ at $opt(time) "stop"
$ns_ at $opt(time).000000002 "puts \"Simulation done.\" ; $ns_ halt"

#start simulation
puts "Starting Simulation..."
$ns_ run
