#!/usr/bin/perl
# This script generates traffic files for the ns2 Network
# Simulator. It works like the CMU tool in 
# ~ns2/indep-utils/cmu-scen-gen/cbrgen.tcl - Unlike cbrgen.tcl it
# generates traffic during your simulationtime only and creates
# exactly as much connections as you said. Additionally you can
# specify for how long data will send on each connection.
# 
# Run it with -h for commandline options and default values
#
# Copyright (C) 2004 Andreas Gohr <andi@splitbrain.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#                         

use Getopt::Std;

#get commandline options
my $opt;
getopts('n:t:c:r:s:p:o:i:l:h',\%opt);
arghelp() if $opt{h};
mkdefault(\%opt);

# initialize the random generator
srand($opt{s});

# create connections
my ($count,$time,$src,$dst);
for ($count=0; $count < $opt{c}; $count++){
  $time = rand($opt{t});
  $src  = int(rand($opt{n}));
  do{
    $dst = int(rand($opt{n}));
  }while($src == $dst);
  if(lc($opt{i}) eq 'tcp'){
    tcp();
  }else{
    cbr();
  }
}


sub cbr(){
 my $stop = $time+$opt{l};
print <<end;
#
# Node $src connecting to Node $dst at time $time
#
set udp_($count) [new Agent/UDP]
\$ns_ attach-agent \$node_($src) \$udp_($count)
set null_($count) [new Agent/Null]
\$ns_ attach-agent \$node_($dst) \$null_($count)
set cbr_($count) [new Application/Traffic/CBR]
\$cbr_($count) set packetSize_ $opt{p}
\$cbr_($count) set interval_ $opt{v}
\$cbr_($count) set random_ 1
\$cbr_($count) set maxpkts_ $opt{o}
\$cbr_($count) attach-agent \$udp_($count)
\$ns_ connect \$udp_($count) \$null_($count)
\$ns_ at $time "\$cbr_($count) start"
\$ns_ at $stop "\$cbr_($count) stop"
end
}

sub tcp(){
 my $stop = $time+$opt{l};
print <<end;
#
# Node $src connecting to Node $dst at time $time
#
set tcp_($count) [\$ns_ create-connection TCP \$node_($src) TCPSink \$node_($dst) 0]
\$tcp_($count) set window_ 32
\$tcp_($count) set packetSize_ $opt{p}
set ftp_($count) [\$tcp_($count) attach-source FTP]
\$ns_ at $time \"\$ftp_($count) start\"
\$ns_ at $stop \"\$ftp_($count) stop\"
end
}

# setup defaults
sub mkdefault($){
  $opt=$_[1];
  $opt{n}=50    if ($opt{n} eq '');
  $opt{t}=100   if ($opt{t} eq '');
  $opt{c}=50    if ($opt{c} eq '');
  $opt{r}=4     if ($opt{r} eq '');
  $opt{s}=time()if ($opt{s} eq '');
  $opt{p}=512   if ($opt{p} eq '');
  $opt{o}=100   if ($opt{o} eq '');
  $opt{l}=20    if ($opt{l} eq '');
  $opt{i}='cbr' if ($opt{i} eq '');
  $opt{v}=1/$opt{r};

  #calculate bitrate
  $kbps= sprintf("%.1f",($opt{r}*$opt{p})/1024);
  #calculate stime
  $stime= sprintf("%.1f",$opt{o}/$opt{r});

  print <<end;
# $opt{c} connections for $opt{n} nodes for a simulation running
# $opt{t} seconds. Traffic is $opt{i} with packets of $opt{p} bytes.
# On each connection data will be send for a maximum of $opt{l}
# seconds. The random generator was seeded with $opt{s}.
#
end

  if($opt{i} eq 'cbr'){
  print <<end;
# The CBR sources send $kbps kilobytes per second ($opt{o} packets
# with $opt{p} bytes). Each will need $stime seconds to send all packets.
#
end
  }

  #make sure to only create traffic completly within simtime
  $opt{t} = $opt{t}-$opt{l};
  die("-t may not be smaller than -l") if($opt{t} < 0);
}

# print help
sub arghelp(){
  print STDERR <<end;
  Usage: trafficgen.pl <options>
  
  Options:
    -n <nodes>            50
    -t <simulationtime>   100 second
    -c <connections>      50
    -r <rate>             4 per second
    -p <packetsize>       512 byte
    -s <randomseed>       current timestamp
    -o <packets>          100
    -l <maxsendtime>      20
    -i <type>             cbr 
    
    -h displays this help
end
  exit 0;
}
